@@ -141,7 +141,7 @@ class CGDB:
         """
         return self._aggregations_manager.aggregations()
 
-    def create_data_set(self, site, element_code, aggregation_code, time_step_code, data_set_type, elementMarkOverride= None, level_code=None):
+    def create_data_set(self, site, element_code, aggregation_code, time_step_code, data_set_type, elementMarkOverride= None, level_code=None, description=None):
         """Create data set
 
              :type site: Site
@@ -155,6 +155,9 @@ class CGDB:
 
              :type time_step_code: str
              :param time_step_code: code of time_step
+
+             :type description: str
+             :param description: code of description
          """
         if self.element_context_by_element(
                 element_code,
@@ -167,7 +170,7 @@ class CGDB:
             else:
                 self.create_element_context(self.element(element_code), aggregation_code,
                                             self.time_step(time_step_code))
-        self._data_sets_manager.create_data_set(site, element_code, aggregation_code, time_step_code, level_code, data_set_type, elementMarkOverride,)
+        self._data_sets_manager.create_data_set(site, element_code, aggregation_code, time_step_code, level_code, data_set_type, elementMarkOverride, description=description)
 
     def create_station(self, name: str, code: str, gpsLatitude: float, gpsLongitude: float, altitude: int, stationNetworkCode:str, stationGroupCode:str = None, remark=""):
         """Create station
