@@ -1,17 +1,47 @@
-import datetime
+from datetime import datetime
 import json
+import pytz
 
 import numpy as np
 import pandas as pd
 from pandas.api.types import is_datetime64_any_dtype as is_datetime
 
-
 from cgdb.exceptions.ColumnNotFound import ColumnNotFoundException
 from cgdb.resources import OverwriteStrategy
-from cgdb.resources.dataSet import DataSets
+from cgdb.resources.dataSet import DataSet
 from cgdb.utils.ManagerMix import ManagerMix
 
 
+def _json_values_to_dataframe(values, mapping_flags, mapping_params, data_set: DataSet):
+    records = []
+    for record in values:
+        record_to_data_frame = {"timestamp": record["timestamp"], "value": record["value"]}
+        if record["flags"] is not None:
+            for flag in record["flags"]:
+                flag_info = mapping_flags[flag]
+                record_to_data_frame[flag_info[0]] = flag_info[1]
+        if record["parameters"] is not None:
+            for param_col, param_value in record["parameters"].items():
+                param_info = mapping_params[param_col]
+                record_to_data_frame[param_info[0]] = param_value
+
+        records.append(record_to_data_frame)
+
+    if len(records) == 0:
+        return pd.DataFrame({"datetime": [], "value": []})
+
+    data_page = pd.DataFrame(records)
+    data_page["timestamp"] = data_page["timestamp"]+3600
+    data_page["timestamp"] = pd.to_datetime(data_page['timestamp'], unit="s", )
+    # data_page.timestamp = data_page.timestamp.dt.tz_localize('Europe/Brussels')
+    data_page.rename(columns={"timestamp": "datetime", "value": data_set.code}, inplace=True)
+    for parameter in data_set.parameters:
+        if parameter.name in data_page.columns and parameter.type in ["INTEGER", "DOUBLE"]:
+            data_page[parameter.name] = pd.to_numeric(data_page[parameter.name])
+
+    return data_page
+
+
 class DataSetsManager(ManagerMix):
     def __init__(self, client):
         super().__init__(client)
@@ -21,11 +51,11 @@ class DataSetsManager(ManagerMix):
         data_sets = []
 
         for station_raw in content:
-            data_sets.append(DataSets(**station_raw, client=self._client, site=site))
+            data_sets.append(DataSet(**station_raw, client=self._client, site=site))
 
         return data_sets
 
-    def data_set_by_code(self, site, element, aggregation, time_step, level = None, elementMarkOverride = None):
+    def data_set_by_code(self, site, element, aggregation, time_step, level=None, elementMarkOverride=None):
         if elementMarkOverride is not None:
             element = elementMarkOverride
         code = element + "_" + aggregation + "_" + str(time_step)
@@ -33,9 +63,73 @@ class DataSetsManager(ManagerMix):
             code += "_" + str(level)
         content = self.get(site.url + "/data-sets/" + code)
 
-        return DataSets(**content, client=self._client, site=site)
+        return DataSet(**content, client=self._client, site=site)
 
-    def get_data_set_data(self, data_set: DataSets, date_from: datetime.datetime = None, date_to: datetime.datetime = None):
+    # def get_data_set_data(self, data_set: DataSet, date_from: datetime.datetime = None, date_to: datetime.datetime = None):
+    #     if (date_from is not None and date_to is None) or (date_from is None and date_to is not None):
+    #         print("one date missing")
+    #         return
+    #
+    #     url = f"{data_set.url}/data"
+    #
+    #     if date_from is not None and date_to is not None:
+    #         date_from = int(date_from.timestamp())
+    #         date_to = int(date_to.timestamp())
+    #
+    #         url_param = f"?date-time-from={date_from}&date-time-to={date_to}&page-size=500"
+    #         url += url_param
+    #
+    #
+    #     content = self.get(url)
+    #     items_count = content["itemsCount"]
+    #     items_count_loaded = 0
+    #     page = 0
+    #     data=None
+    #     # load data from pages
+    #     while items_count_loaded < items_count:
+    #         if items_count_loaded == 0:
+    #             values = content["values"]
+    #         else:
+    #             values = self.get(url+"&page="+str(items_count_loaded))["values"]
+    #
+    #         mapping_flags = data_set._mapping_flags_to_value()
+    #         mapping_params = data_set._mapping_parameters_to_col_name_type()
+    #
+    #         items_count_loaded += len(values)
+    #         page += 1
+    #         records = []
+    #         for record in values:
+    #             record_to_data_frame = {"timestamp": record["timestamp"], "value": record["value"]}
+    #             if record["flags"] is not None:
+    #                 for flag in record["flags"]:
+    #                     flag_info = mapping_flags[flag]
+    #                     record_to_data_frame[flag_info[0]] = flag_info[1]
+    #             if record["parameters"] is not None:
+    #                 for param_col, param_value in record["parameters"].items():
+    #                     param_info = mapping_params[param_col]
+    #                     record_to_data_frame[param_info[0]] = param_value
+    #
+    #             records.append(record_to_data_frame)
+    #
+    #         if data is None and len(records) == 0:
+    #             return pd.DataFrame({"datetime": [], "value":[]})
+    #
+    #         data_page = pd.DataFrame(records)
+    #         data_page["timestamp"] = pd.to_datetime(data_page['timestamp'], unit="s")
+    #         # data_page.timestamp = data_page.timestamp.dt.tz_localize('UTC').dt.tz_convert('Europe/Brussels')
+    #         data_page.rename(columns={"timestamp": "datetime", "value": data_set.code}, inplace=True)
+    #         for parameter in data_set.parameters:
+    #             if parameter.name in data_page.columns and parameter.type in ["INTEGER", "DOUBLE"]:
+    #                 data_page[parameter.name] = pd.to_numeric(data_page[parameter.name])
+    #
+    #         if data is None:
+    #             data = data_page
+    #         else:
+    #             data = pd.concat([data, data_page])
+    #
+    #     return data
+
+    def get_data_set_data2(self, data_set: DataSet, date_from: str = None, date_to: str = None):
         if (date_from is not None and date_to is None) or (date_from is None and date_to is not None):
             print("one date missing")
             return
@@ -43,59 +137,48 @@ class DataSetsManager(ManagerMix):
         url = f"{data_set.url}/data"
 
         if date_from is not None and date_to is not None:
-            date_from = int(date_from.timestamp())
-            date_to = int(date_to.timestamp())
 
-            url_param = f"?date-time-from={date_from}&date-time-to={date_to}&page-size=500"
+            date_from = int(datetime
+                            .strptime(date_from, "%Y-%m-%d %H:%M")
+                            .replace(tzinfo=pytz.timezone("UTC")).timestamp()) - 3600
+            # date_from = date_from - (date_from % 300)
+            date_to = int(datetime
+                          .strptime(date_to, "%Y-%m-%d %H:%M")
+                          .replace(tzinfo=pytz.timezone("UTC")).timestamp()) - 3600
+            # date_to = date_to - (date_to % 300)
+
+            url_param = f"?date-time-from={date_from}&page-size=10000"
             url += url_param
 
-
-        content = self.get(url)
+        content = self.get(url + f"&date-time-to={date_to}")
+        print(url + f"&date-time-to={date_to}")
+        values = content["values"]
         items_count = content["itemsCount"]
-        items_count_loaded = 0
-        page = 0
-        data=None
-        # load data from pages
-        while items_count_loaded < items_count:
-            if items_count_loaded == 0:
-                values = content["values"]
-            else:
-                values = self.get(url+"&page="+str(items_count_loaded))["values"]
+        print("items count:" + str(items_count))
 
-            mapping_flags = data_set._mapping_flags_to_value()
-            mapping_params = data_set._mapping_parameters_to_col_name_type()
+        mapping_flags = data_set._mapping_flags_to_value()
+        mapping_params = data_set._mapping_parameters_to_col_name_type()
 
-            items_count_loaded += len(values)
-            page += 1
-            records = []
-            for record in values:
-                record_to_data_frame = {"timestamp": record["timestamp"], "value": record["value"]}
-                if record["flags"] is not None:
-                    for flag in record["flags"]:
-                        flag_info = mapping_flags[flag]
-                        record_to_data_frame[flag_info[0]] = flag_info[1]
-                if record["parameters"] is not None:
-                    for param_col, param_value in record["parameters"].items():
-                        param_info = mapping_params[param_col]
-                        record_to_data_frame[param_info[0]] = param_value
+        data = _json_values_to_dataframe(values, mapping_flags, mapping_params, data_set)
 
-                records.append(record_to_data_frame)
+        items_count_loaded = data.shape[0]
+        items_count_loaded_left = items_count - items_count_loaded
 
-            if data is None and len(records) == 0:
-                return pd.DataFrame({"datetime": [], "value":[]})
+        while items_count_loaded_left > 0:
+            date_to = values[-1]["timestamp"] - 1
 
-            data_page = pd.DataFrame(records)
-            data_page["timestamp"] = pd.to_datetime(data_page['timestamp'], unit="s")
-            # data_page.timestamp = data_page.timestamp.dt.tz_localize('UTC').dt.tz_convert('Europe/Brussels')
-            data_page.rename(columns={"timestamp": "datetime", "value": data_set.code}, inplace=True)
-            for parameter in data_set.parameters:
-                if parameter.name in data_page.columns and parameter.type in ["INTEGER", "DOUBLE"]:
-                    data_page[parameter.name] = pd.to_numeric(data_page[parameter.name])
+            content = self.get(url + f"&date-time-to={date_to}")
+            print(url + f"&date-time-to={date_to}")
+            print("items count:" + str(content["itemsCount"]))
+            values = content["values"]
 
-            if data is None:
-                data = data_page
-            else:
-                data = pd.concat([data, data_page])
+            data_tmp = _json_values_to_dataframe(values, mapping_flags, mapping_params, data_set)
+            items_count_loaded = data.shape[0]
+            items_count_loaded_left = items_count_loaded_left - items_count_loaded
+
+            data = pd.concat([data, data_tmp])
+
+        # data['datetime'] = data['datetime'].dt.tz_localize('utc').dt.tz_convert("Europe/Prague")
 
         return data
 
@@ -111,7 +194,7 @@ class DataSetsManager(ManagerMix):
     #
     #     return pd.DataFrame(columns)
 
-    def post_data_set_data2(self, data: pd.DataFrame, data_set: DataSets, overwriteStrategy: str = None):
+    def post_data_set_data2(self, data: pd.DataFrame, data_set: DataSet, overwriteStrategy: str = None):
         print(data)
 
         if "date_time" not in data.columns:
@@ -121,16 +204,21 @@ class DataSetsManager(ManagerMix):
 
         data = data.dropna(subset=["value"])
 
-        parameters_prefix_columns_data = [column_name for column_name in data.columns if column_name.startswith("parameter-")]
+        parameters_prefix_columns_data = [column_name for column_name in data.columns if
+                                          column_name.startswith("parameter-")]
         flags_prefix_columns_data = [column_name for column_name in data.columns if column_name.startswith("flag-")]
 
         data_set_parameters_names = [parameter.name for parameter in data_set.parameters]
         data_set_flags_names = [flag.name for flag in data_set.flags]
 
-        if not all(column in data_set_parameters_names for column in [column_with_prefix[len("parameter-"):] for column_with_prefix in parameters_prefix_columns_data]):
-            raise Exception("parameters columns are not in data set. columns: " + str(parameters_prefix_columns_data) + " in data set: " + str(data_set_parameters_names))
-        if not all(column in data_set_flags_names for column in [column_with_prefix[len("flag-"):] for column_with_prefix in flags_prefix_columns_data]):
-            raise Exception("flags columns are not in data set. columns: " + str(flags_prefix_columns_data) + " in data set: " + str(data_set_flags_names))
+        if not all(column in data_set_parameters_names for column in
+                   [column_with_prefix[len("parameter-"):] for column_with_prefix in parameters_prefix_columns_data]):
+            raise Exception("parameters columns are not in data set. columns: " + str(
+                parameters_prefix_columns_data) + " in data set: " + str(data_set_parameters_names))
+        if not all(column in data_set_flags_names for column in
+                   [column_with_prefix[len("flag-"):] for column_with_prefix in flags_prefix_columns_data]):
+            raise Exception("flags columns are not in data set. columns: " + str(
+                flags_prefix_columns_data) + " in data set: " + str(data_set_flags_names))
 
         data = data[["date_time", "value"] + parameters_prefix_columns_data + flags_prefix_columns_data].copy()
 
@@ -147,14 +235,16 @@ class DataSetsManager(ManagerMix):
                     data[parameter_column_data] = data[parameter_column_data].astype('Int64')
                 except TypeError:
                     raise Exception("Paramater column:" + parameter_column_data + " must be integer")
-            elif parameters_mapping[parameter_column_data][1] == "DOUBLE" and data.dtypes[parameter_column_data] != np.float64:
+            elif parameters_mapping[parameter_column_data][1] == "DOUBLE" and data.dtypes[
+                parameter_column_data] != np.float64:
                 raise Exception("Paramater column:" + parameter_column_data + " must be Double")
-            data.rename(columns={parameter_column_data: str(parameters_mapping[parameter_column_data][0])}, inplace=True)
+            data.rename(columns={parameter_column_data: str(parameters_mapping[parameter_column_data][0])},
+                        inplace=True)
             parameters_id_columns_data.append(str(parameters_mapping[parameter_column_data][0]))
 
         if is_datetime(data["date_time"]):
             data['date_time'] = data.date_time.dt.tz_localize(tz="UTC")
-            data["date_time"] = (data.date_time.values.astype(np.int64) / 10**9).astype(np.int64) - 3600
+            data["date_time"] = (data.date_time.values.astype(np.int64) / 10 ** 9).astype(np.int64) - 3600
 
         out = []
         for index, row in data.iterrows():
@@ -179,7 +269,6 @@ class DataSetsManager(ManagerMix):
 
         out = json.dumps(out)
 
-
         url = data_set.url + "/data"
 
         if overwriteStrategy is not None:
@@ -187,9 +276,7 @@ class DataSetsManager(ManagerMix):
 
         a = self._client._session.post(url, data=out)
         if a.status_code != 200:
-            raise Exception("upload fail on server side "+a.text)
-
-
+            raise Exception("upload fail on server side " + a.text)
 
     # def post_data_set_data(self, url, data: pd.DataFrame, date_time_column_name:str = "date_time", value_column_name:str = "value", parameter_columns_names:dict = None, flags_columns_names:dict = None, flags=None, parameters=None):
     #     url = f"{url}/data"
@@ -274,8 +361,10 @@ class DataSetsManager(ManagerMix):
     #
     #     return json.dumps(out)
 
-    def create_data_set(self, station, element_code, aggregation, time_step_code, level_code, data_set_type="TECHNICAL", elementMarkOverride = None, description = None):
-        element_context = self._client._element_contexts_manager.element_context_by_element(element_code, aggregation, time_step_code, level_code)
+    def create_data_set(self, station, element_code, aggregation, time_step_code, level_code, data_set_type="TECHNICAL",
+                        elementMarkOverride=None, description=None):
+        element_context = self._client._element_contexts_manager.element_context_by_element(element_code, aggregation,
+                                                                                            time_step_code, level_code)
 
         url = station.url + "/data-sets"
 
@@ -286,5 +375,5 @@ class DataSetsManager(ManagerMix):
 
         self.post(url, json.dumps(content))
 
-    def delete_data_set(self, data_set: DataSets):
+    def delete_data_set(self, data_set: DataSet):
         self.delete(data_set.url)
